import numpy as np
import matplotlib.pyplot as plt
import matplotlib
# matplotlib.use('TkAgg')
from pathlib import Path
import h5py
import os
import sfHMM
import warnings

import warnings
from sklearn.exceptions import ConvergenceWarning
warnings.filterwarnings(action='ignore', category=ConvergenceWarning)
baseDir = Path('M:/tnw/bn/cd/Shared/Tisma/ParB-BsSMC_spt/2023/20230508_hdf5_Lipfert_coil')
dataFolderPlus = baseDir / '(+)coil' / '200ms_levels' / 'all'
dataFolderMin = baseDir / '(+)coil' / '200ms_levels' / 'all'

assert(dataFolderMin.exists())
assert(dataFolderPlus.exists())

hfd5FilesPlus = []

for f in dataFolderPlus.iterdir():
    if f.suffix == '.hdf5':
        hfd5FilesPlus.append(f)
print(f'Found {len(hfd5FilesPlus)} files to analyse')

ratiosPlus = []
for i, p in enumerate(hfd5FilesPlus):
    with h5py.File(p, 'r') as f:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            kymoLeft = f['Left Kymograph Loop'][()]
            sf = sfHMM.sfHMM1(np.sum(kymoLeft, axis=0), krange=(2, 2))
            sf.step_finding()
            sf.denoising()
            sf.gmmfit()
            sf.hmmfit()
            ratiosPlus.append(np.round(min(sf.gmm.get_optimal().means_) / max(sf.gmm.get_optimal().means_), 3)[0])
print(ratiosPlus)

hfd5FilesMin = []

for f in dataFolderMin.iterdir():
    if f.suffix == '.hdf5':
        hfd5FilesMin.append(f)
print(f'Found {len(hfd5FilesMin)} files to analyse')

ratiosMin = []
for i, p in enumerate(hfd5FilesMin):
    with h5py.File(p, 'r') as f:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            kymoLeft = f['Left Kymograph Loop'][()]
            sf = sfHMM.sfHMM1(np.sum(kymoLeft, axis=0), krange=(2, 2))
            sf.step_finding()
            sf.denoising()
            sf.gmmfit()
            sf.hmmfit()
            ratiosMin.append(np.round(max(sf.gmm.get_optimal().means_) / min(sf.gmm.get_optimal().means_), 3)[0])
print(ratiosMin)

plt.figure()
bins = np.linspace(0, 3, 61)
nm, bm, pm = plt.hist(ratiosMin, alpha=1, bins=bins, label='(-)', rwidth=0.8, density=False)  # To make me a density plot, do: density=True
npl, bp, pp = plt.hist(ratiosPlus, alpha=1, bins=bins, label='(+)', rwidth=0.8, density=False)
plt.xlabel('Total intensity ratio coiled/nicked')
plt.ylabel('Count')
plt.xlim(0, 3.4)
plt.legend()

print(f'Median ratio of negative supercoils: {np.median(ratiosMin)}\n'
      f'Median ratio of positive supercoils: {np.median(ratiosPlus)}\n'
      f'Mean ratio of negative supercoils: {np.mean(ratiosMin)}\n'
      f'Mean ratio of positive supercoils: {np.mean(ratiosPlus)}\n'
      f'Std of negative supercoils: {np.std(ratiosMin)}\n'
      f'Std of positive supercoils: {np.std(ratiosPlus)}\n')

with open('(+)coil_200ms_levels_all-RATIOS.txt', 'w') as file:
    file.write('\n'.join(np.array(ratiosPlus).astype(str)))
with open('(-)coil_200ms_levels_all-RATIOS.txt', 'w') as file:
    file.write('\n'.join(np.array(ratiosMin).astype(str)))
with open('(-)coil_bincounts.txt', 'w') as file:
    file.write('\n'.join(np.array(nm).astype(str)))
with open('(+)coil_bincounts.txt', 'w') as file:
    file.write('\n'.join(np.array(npl).astype(str)))
with open('bins.txt', 'w') as file:
    file.write('\n'.join(np.array(bins).astype(str)))

plt.show()
